#!/usr/bin/env python3
import os
import sys
import git
import markdown2
from lxml import etree
from os import listdir, makedirs
from os.path import isfile, isdir, join, exists

def xmlValidation(lesson_path,schema_path):
    """
    Will perform the validation of an .xml file against a RNG schema.
    Will return true if the file is validated.
    """
    ## Will get the content of our new .xml file to be validated
    doc = etree.parse(lesson_path)
    ## Will parse our schema
    relaxng_doc = etree.parse(schema_path)
    ## Will ingest the schema as an RNG schema
    relaxng = etree.RelaxNG(relaxng_doc)
    ## Checking if the document is indeed validated against the given schema
    validated = relaxng.validate(doc)
    log = relaxng.error_log
    if(not validated):
        # print("Error : Invalid file :"+lesson_path+" against schema. See log below\n")
        print(log.last_error)
    return(validated)

def HTMLtoXMLparser_XSLT(lesson_path,lesson_name,lesson_part,html):
    """
    Will perform the parsing and the tranformation from html to xml.
    Will also call the validation function to validate the generated .xml files.
    """
    tree = etree.HTML(html)

    # create directory if it doesn't exist
    directory="data/"+lesson_path+lesson_name

    # We need to create the directory if it doesn't exist yet
    if not os.path.exists(directory):
        os.makedirs(directory)

    ## Unfortunately, it's not yet possible to use a file for the XML function so... we use this long string
    xslt = etree.XML('''\
        <xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
            <xsl:output method="xml" indent="yes"/>

            <xsl:template match="/">
                <lesson_part>
                    <title>
                        <xsl:value-of select="//h1"/>
                    </title>
                    <lessonCore>
                        <xsl:apply-templates select="//h2/following-sibling::*[count(preceding::h2) = 1 and text()!='Exercise']"/>
                    </lessonCore>
                    <exercise>
                        <xsl:apply-templates select="//h2/following-sibling::*[count(preceding::h2) = 2 and text()!='Quiz Question']"/>
                    </exercise>
                    <quiz>
                    <question>
                        <xsl:apply-templates select="//h2/following-sibling::*[count(preceding::h2) = 3 and text()!='Quiz Answer']"/>
                    </question>
                    <answer>
                        <xsl:apply-templates select="//h2/following-sibling::*[count(preceding::h2) = 4]"/>
                    </answer>
                    </quiz>
                </lesson_part>
            </xsl:template>

            <xsl:template match="//pre">
                <listing>
                    <xsl:value-of select="."/>
                </listing>
            </xsl:template>

            <xsl:template match="//p[./text()]">
                <paragraph>
                    <xsl:apply-templates/>
                </paragraph>
            </xsl:template>

            <xsl:template match="//p/b">
                <emphasis><xsl:value-of select="."/></emphasis>
            </xsl:template>

            <xsl:template match="//p[not(./text())]/b">
                <subtitle>
                    <xsl:value-of select="."/>
                </subtitle>
            </xsl:template>

            <xsl:template match="//ul">
                <unorderedList>
                    <xsl:apply-templates/>
                </unorderedList>
            </xsl:template>

            <xsl:template match="//ol">
                <orderedList>
                    <xsl:apply-templates/>
                </orderedList>
            </xsl:template>

            <xsl:template match="//a">
                <xsl:element name="link">
                    <xsl:attribute name="href">
                        <xsl:value-of select="./@href"/>
                    </xsl:attribute>
                    <xsl:value-of select="."/>
                </xsl:element>
            </xsl:template>

            <xsl:template match="//ul/li | //ol/li">
                <listElement>
                    <xsl:apply-templates/>
                </listElement>
            </xsl:template>

        </xsl:stylesheet>
        ''')

    ## Will initiate the transform function with the xslt
    transform = etree.XSLT(xslt)
    ## Will perform the transformation
    result = transform(tree)
    ## Will write the generated xml file in the right folder
    result.write_output(directory+"/"+lesson_part+".xml")
    # print(result)

    ## In case we want the associated html files
    # f= open(directory+"/"+lesson_part+".html","w")
    # f.write(html)
    # f.close()

def getLessons(lesson_path,language_lessons_path,schema_path):
    """
    Get and transform the .md files to generate and validate the final .xml files
    """
    ## Get the path to the lessons .md files
    lessons_path = lesson_path+language_lessons_path

    ## Get the list of all the lessons directories to be able to iterate through them
    lessons_directories = [f for f in listdir(lessons_path) if isdir(join(lessons_path, f))]

    ## Initializing the counters for the validation check
    total_valid_lessons = 0
    total_number_of_lessons = 0

    for lesson_name in lessons_directories:
        print("########## Generating a new lesson : "+lesson_name+" ##########")
        ## Get the list of all the lessons .md files to be able to iterate through them
        lessons = [f for f in listdir(lessons_path+lesson_name) if isfile(join(lessons_path+lesson_name, f))]

        ## The number of valid lessons of this directory should be reseted to 0 at each iteration
        valid_lessons=0

        for lesson_part in lessons:
            total_number_of_lessons+=1
            ## Will transform the markdown file content to html
            html = markdown2.markdown_path(lessons_path+lesson_name+"/"+lesson_part)
            ## Only the name of the lesson part is needed, the extension will then change from .md to .xml
            lesson_part_without_extension = os.path.splitext(lesson_part)[0]
            ## Will perform the parsing and conversion of the current .md lesson part into .xml files
            HTMLtoXMLparser_XSLT(language_lessons_path,lesson_name,lesson_part_without_extension,html)
            ## Will perform a validation check against the current schema
            if(xmlValidation("data/"+language_lessons_path+lesson_name+"/"+lesson_part_without_extension+".xml",schema_path)):
                valid_lessons+=1
                total_valid_lessons+=1
            # break
        print("Valid lessons : "+str(valid_lessons)+"/"+str(len(lessons)))
    print("Total valid lessons : "+str(total_valid_lessons)+"/"+str(total_number_of_lessons))

def pullFromRepo():
    """
    TODO : Even though this is not fully implemented yet, it would be nice to have this option available
    in order to get the repo updated whenever we want it, automatically.
    """
    # print("---- INITIATING PULL ----")
    # DIR_NAME = "../"
    # REMOTE_URL = "https://github.com/cindyq/linuxjourney"
    # repo = git.Repo.init(DIR_NAME)
    # origin = repo.create_remote('origin',REMOTE_URL)
    # origin.fetch()
    # origin.pull(origin.refs[0].remote_head)
    # print("---- DONE ----")

def main(argv):
    """
    Will run the parser, generate the wanted xml file.
    """
    if(len(argv)>0):
        if(argv[0]=="--pull"):
            pullFromRepo()

    ## Path variables
    lesson_path="../lessons/locales/"
    english_lessons_path="en_english/"
    french_lessons_path="fr_french/"
    schema_path="models/linuxjourney-schema.rng"

    ## Get, transform, and create the validated files for the english lessons
    englishLessons = getLessons(lesson_path,english_lessons_path,schema_path)

    ## Get, transform, and create the validated files for the french lessons
    # frenchLessons = getLessons(lesson_path,french_lessons_path,schema_path)

if __name__ == '__main__':
    main(sys.argv[1:])
